//
// Chobits OS  ҽ  .
//


#include "../../ddk.h"

/*
 * global function
 */
GLOBAL_FUNCTION( snake_main )

/*
 * definitions
 */
#define BORDER_LEFT_POS		0
#define BORDER_TOP_POS		1
#define BORDER_RIGHT_POS	(30-1-BORDER_LEFT_POS)
#define BORDER_BOTTOM_POS	(10-2)

#define DIRECTION_UP		0x00
#define DIRECTION_DOWN		0x01
#define DIRECTION_LEFT		0x02
#define DIRECTION_RIGHT		0x03

#define MSG_BOX_XPOS		5
#define MSG_BOX_YPOS		(BORDER_BOTTOM_POS+1)

#define LEFT_KEY			'j'
#define RIGHT_KEY			'l'
#define UP_KEY				'i'
#define DOWN_KEY			'k'

#define RETRY_KEY			'r'
#define QUIT_KEY			'q'

#define APPLE_PLUS_CELL		2
#define SNAKE_TOTAL_CELL	100

#define TIME_DELAY			200 /* 0.1 sec */

/*
 * structures
 */
typedef struct _SNAKE_CELL {
	BOOL		draw;
	WORD		x, y;
	BYTE		direction;
} SNAKE_CELL, *PSNAKE_CELL;

typedef struct _APPLE_POS {
	WORD		x, y;
} APPLE_POS, *PAPPLE_POS;

/*
 * internal functions
 */
static void draw_border(void);
static void draw_apple(void);
static BOOL move_snake(void);
static void move_snake_cell(SNAKE_CELL *pCell);
static void draw_snake_with_this(BYTE *pThis);
static void print_score(void);
static void got_apple(void);
static void retry_game(void);
static BOOL is_snake_corrupt(void);
static BOOL lengthen_snake(BYTE new_pos);

static BOOL get_key(BYTE *pKey);

/*
 * external data
 */
extern HANDLE			m_IntroTaskHandle;

/*
 * global variables
 */
static BYTE		body[2];
static BYTE		apple[2];
static BYTE		block[2];
static BYTE		blank[2];

static SNAKE_CELL	m_SnakeCell[SNAKE_TOTAL_CELL];
static APPLE_POS	m_ApplePos;
static BYTE			m_CurDirection;
static DWORD		m_Score;
static BYTE			m_RemainedLength;

#define API_PrintText	DbgPrint
#define API_PrintTextXY DbgPrintXY


//////////////////////////////////////////////////////////////////////////////////////////
void API_ClearScreen()
{
	ULONG	i;
	PUSHORT pt_video = (PUSHORT)GBA_DISP_VMEM_ADDR;
	
	for(i=0; i<(GBA_DISP_WIDTH*(GBA_DISP_HEIGHT-16)); i++)
	{
		pt_video[i] = 0;
	}
}

VOID
snake_main(PVOID pContext)
{
	BYTE key_data;
	
	GBA_KEY_REG_CNT = 0x43ff;
	
	body[0]  = '#';  body[1] = 0x00;
	apple[0] = 0x1; apple[1] = 0x00;
	block[0] = '*'; block[1] = 0x00;
	blank[0] = ' '; blank[1] = 0x00;
	
	// wait intro task
	KeWaitForSingleObject( m_IntroTaskHandle, INFINITE );

	/* init */
	retry_game();

	while(1)
	{
		/* get key */
		if(get_key(&key_data))
		{
			switch(key_data)
			{
				case LEFT_KEY:
					m_CurDirection = DIRECTION_LEFT;
					break;
				case RIGHT_KEY:
					m_CurDirection = DIRECTION_RIGHT;
					break;
				case UP_KEY:
					m_CurDirection = DIRECTION_UP;
					break;
				case DOWN_KEY:
					m_CurDirection = DIRECTION_DOWN;
					break;
				case QUIT_KEY:
					goto $exit;
				default:
					break;
			}
		}

		/* score */
		print_score();

		/* draw apple */
		draw_apple();

		/* draw snake */
		if(!move_snake())
		{
			DbgColorPrint(RGB(255,0,0), RGB(0,0,255), "  U DEAD! Press 'A' to retry  ");
			while(1)
			{
				while( !get_key(&key_data) )
				{
					KeSleep(10);
				}
				if( key_data != RETRY_KEY )
				{
					continue;
				}
				break;
			}
			retry_game();
			continue;
		}

		/* delay */
		KeSleep(TIME_DELAY);
	}

$exit:
	//API_FlushKbdBuf();
	//API_ClearScreen();
	API_PrintText("Bye~! ;) \r\n");
	while(1); 
}

static void retry_game(void)
{
	int i;

	memset(&m_ApplePos, 0, sizeof(m_ApplePos));
	memset(m_SnakeCell, 0, sizeof(m_SnakeCell));

	m_Score = 0;
	m_RemainedLength = 0;
	m_CurDirection = DIRECTION_RIGHT;

	m_ApplePos.x = BORDER_LEFT_POS+5;
	m_ApplePos.y = BORDER_TOP_POS+5;

	m_SnakeCell[0].draw = TRUE;
	m_SnakeCell[0].x = (BORDER_RIGHT_POS+BORDER_LEFT_POS)/2;
	m_SnakeCell[0].y = (BORDER_BOTTOM_POS+BORDER_TOP_POS)/2;
	m_SnakeCell[0].direction = DIRECTION_RIGHT;
	for(i=1; i<2; i++) {
		m_SnakeCell[i].draw = TRUE;
		m_SnakeCell[i].x = m_SnakeCell[i-1].x-1;
		m_SnakeCell[i].y = (BORDER_BOTTOM_POS+BORDER_TOP_POS)/2;
		m_SnakeCell[i].direction = DIRECTION_RIGHT;
	}

	/* draw border */
	draw_border();
}

static BOOL get_key(BYTE *pKey)
{
	USHORT key_val = GBA_KEY_REG_FLAG;
	
	if( !(key_val & 0x01) )
	{
		*pKey = RETRY_KEY;
	}
	else if( !(key_val & 0x10) )
	{
		*pKey = RIGHT_KEY;
	}
	else if( !(key_val & 0x20) )
	{
		*pKey = LEFT_KEY;
	}
	else if( !(key_val & 0x40) )
	{
		*pKey = UP_KEY;
	}
	else if( !(key_val & 0x80) )
	{
		*pKey = DOWN_KEY;
	}
	else
	{
		return FALSE;
	}
	
	return TRUE;
}

static void print_score(void)
{
#define SCORE_X_POS		0
#define SCORE_Y_POS		0

	BYTE score_str[100];

	sprintf(score_str, "You got : %d points! ;)", m_Score);
	API_PrintTextXY(score_str, SCORE_X_POS, SCORE_Y_POS);
}

static void draw_apple(void)
{
	API_PrintTextXY(apple, m_ApplePos.x, m_ApplePos.y);
}

static BOOL move_snake(void)
{
	int i;
	BYTE new_direction = m_CurDirection, temp;

	/* draw blank */
	draw_snake_with_this(blank);

	/* move pos */
	for(i=0; i<SNAKE_TOTAL_CELL; i++) {
		if(m_SnakeCell[i].draw == FALSE)
			break;

		temp = m_SnakeCell[i].direction;
		m_SnakeCell[i].direction = new_direction;
		move_snake_cell(&m_SnakeCell[i]);
		new_direction = temp;
	}

	/* lengthen */
	if(m_RemainedLength) {
		if(!lengthen_snake(i))
			m_RemainedLength = 0;
		else 
			m_RemainedLength--;
	}

	if(is_snake_corrupt())
		return FALSE;

	/* is out of border? */
	if(m_SnakeCell[0].x <= BORDER_LEFT_POS || m_SnakeCell[0].x >= BORDER_RIGHT_POS
		|| m_SnakeCell[0].y <= BORDER_TOP_POS || m_SnakeCell[0].y >= BORDER_BOTTOM_POS)
		return FALSE;

	/* got apple? */
	if(m_SnakeCell[0].x == m_ApplePos.x && m_SnakeCell[0].y == m_ApplePos.y) {
		got_apple();
		m_RemainedLength+=APPLE_PLUS_CELL;
	}

	/* redraw */
	draw_snake_with_this(body);

	return TRUE;
}

static BOOL lengthen_snake(BYTE new_pos)
{
	int x, y;

	if(new_pos == SNAKE_TOTAL_CELL) {
		return FALSE;
	}

	m_SnakeCell[new_pos].draw = TRUE;
	m_SnakeCell[new_pos].direction = m_SnakeCell[new_pos-1].direction;
	x = m_SnakeCell[new_pos-1].x;
	y = m_SnakeCell[new_pos-1].y;
	switch(m_SnakeCell[new_pos-1].direction) {
		case DIRECTION_UP:
			y++;
			break;
		case DIRECTION_DOWN:
			y--;
			break;
		case DIRECTION_LEFT:
			x++;
			break;
		case DIRECTION_RIGHT:
			x--;
			break;
	}
	m_SnakeCell[new_pos].x = x;
	m_SnakeCell[new_pos].y = y;

	return TRUE;
}

static BOOL is_snake_corrupt(void)
{
	int i, j;

	for(i=0; i<SNAKE_TOTAL_CELL-1; i++) {
		if(m_SnakeCell[i].draw == FALSE)
			break;

		for(j=i+1; j<SNAKE_TOTAL_CELL; j++) {
			if(m_SnakeCell[j].draw == FALSE)
				break;

			if(m_SnakeCell[i].x == m_SnakeCell[j].x && m_SnakeCell[i].y == m_SnakeCell[j].y)
				return TRUE;
		}
	}

	return FALSE;
}

static void got_apple(void)
{
	DWORD tick;
	BYTE new_xpos, new_ypos;

	m_Score += 100;
	print_score();

	while(1) {
		tick = KeQueryTickCount()/10;
		new_xpos = (BYTE)(tick%30);
		new_ypos = (BYTE)(tick%10);

		if(new_xpos <= BORDER_LEFT_POS || new_xpos >= BORDER_RIGHT_POS
			|| new_ypos <= BORDER_TOP_POS || new_ypos >= BORDER_BOTTOM_POS)
			continue;

		for(tick=0; tick<SNAKE_TOTAL_CELL; tick++) {
			if(m_SnakeCell[tick].draw == FALSE)
				break;

			if(m_SnakeCell[tick].x == new_xpos && m_SnakeCell[tick].y == new_ypos)
				goto $go_ahead;
		}

		m_ApplePos.x = new_xpos;
		m_ApplePos.y = new_ypos;
		break;
$go_ahead:
		;
	}

	draw_apple();
}

static void move_snake_cell(SNAKE_CELL *pCell)
{
	switch(pCell->direction)
	{
	case DIRECTION_LEFT:
		pCell->x--;
		break;
	case DIRECTION_UP:
		pCell->y--;
		break;
	case DIRECTION_RIGHT:
		pCell->x++;
		break;
	case DIRECTION_DOWN:
		pCell->y++;
		break;
	}
}

static void draw_snake_with_this(BYTE *pThis)
{
	int i;

	for(i=0; i<SNAKE_TOTAL_CELL; i++)
	{
		if( pThis == body )
		{
			if(m_SnakeCell[i].draw && i==0)
			{
				API_PrintTextXY(pThis, m_SnakeCell[i].x, m_SnakeCell[i].y);
				break;
			}
		}
		else // blank
		{
			if(m_SnakeCell[i].draw)
			{
				if( (i<(SNAKE_TOTAL_CELL-1) && !m_SnakeCell[i+1].draw) )
				{
					API_PrintTextXY(pThis, m_SnakeCell[i].x, m_SnakeCell[i].y);
					break;
				}
			}
		}
	}
}

static void draw_border(void)
{
	BYTE i;

	/* clear screen */
	API_ClearScreen();

	/* draw border */
	for(i=BORDER_LEFT_POS; i<=BORDER_RIGHT_POS; i++) {
		API_PrintTextXY(block, i, BORDER_TOP_POS);
		API_PrintTextXY(block, i, BORDER_BOTTOM_POS);
	}
	for(i=BORDER_TOP_POS; i<=BORDER_BOTTOM_POS; i++) {
		API_PrintTextXY(block, BORDER_LEFT_POS, i);
		API_PrintTextXY(block, BORDER_RIGHT_POS, i);
	}
}
