#ifndef _DDK_HEADER_FILE_
#define _DDK_HEADER_FILE_


/*
 * include files
 */
#include "types.h"
#include "gbadef.h"
#include "string.h"
#include "vsprintf.h"

#include "krnobj.h"


/*
 * definition
 */
#define GMC_KERNEL_VERSION			"1.0"
 
#define GLOBAL_FUNCTION(func)


//////////////////////////////////////////////////////////////////////////////////////////
//                                     Debugging API                                    //
//////////////////////////////////////////////////////////////////////////////////////////
KERNELAPI
BOOL
DbgSetDebugVideoMode(VOID);

KERNELAPI
BOOL
DbgClearVideoScreen(VOID);

KERNELAPI
BOOL
DbgPrint(
	IN	PSTR							pStr
	);
	
KERNELAPI
BOOL
DbgColorPrint(
	IN	USHORT							TextColor,
	IN	USHORT							BkColor,
	IN	PSTR							pStr
	);

KERNELAPI
BOOL
DbgPrintXY(
	IN	PSTR							pStr,
	IN	ULONG							XPos,
	IN	ULONG							YPos
	);
	
KERNELAPI
BOOL
DbgColorPrintXY(
	IN	USHORT							TextColor,
	IN	USHORT							BkColor,
	IN	PSTR							pStr,
	IN	ULONG							XPos,
	IN	ULONG							YPos
	);

//////////////////////////////////////////////////////////////////////////////////////////
//                                        HAL API                                       //
//////////////////////////////////////////////////////////////////////////////////////////
#define ENABLE_INTERRUPTS()				(GBA_INT_REG_IME = 1)
#define DISABLE_INTERRUPTS()			(GBA_INT_REG_IME = 0)

#define IS_INT_IME_ENABLED()			(GBA_INT_REG_IME ? TRUE : FALSE);

typedef VOID (*PIRQ_HANDLER)(VOID);
KERNELAPI
BOOL
HalInstallIrqHandler(
	IN	ULONG							IrqNumber,
	IN	PIRQ_HANDLER					pIrqHandler,		// can be NULL when delete!
	OUT	PIRQ_HANDLER					*ppOldIrqHandler	// optional
	);
	
//////////////////////////////////////////////////////////////////////////////////////////
//                                      Kernel API                                      //
//////////////////////////////////////////////////////////////////////////////////////////
KERNELAPI
ULONG // 1000 : 1sec
KeQueryTickCount(VOID);

KERNELAPI
VOID
KeSleep(
	IN	ULONG							SleepTime // milli seconds
	);
	
//
typedef ULONG				KE_WAIT_STATUS;
#define KE_WAIT_FAILED		0
#define KE_WAIT_SUCCESS		1
#define KE_WAIT_TIMEOUT		2

KERNELAPI
KE_WAIT_STATUS
KeWaitForSingleObject(
	IN	PVOID							pObject,
	IN	ULONG							Timeout // milli seconds
	);

//
KERNELAPI
VOID
KeInitializeMutex(
	IN	PKMUTEX							pMutexObj
	);

KERNELAPI
BOOL
KeReadStateMutex(
	IN	PKMUTEX							pMutexObj,
	OUT	PULONG							pSignalState
	);
	
KERNELAPI
VOID
KeReleaseMutex(
	IN	PKMUTEX							pMutexObj,
	IN	BOOL							bWait
	);
	
//
KERNELAPI
VOID
KeInitializeSemaphore(
	IN	PKSEMAPHORE						pSemaphoreObj,
	IN	ULONG							Count,
	IN	ULONG							Limit
	);

KERNELAPI
BOOL
KeReadStateSemaphore(
	IN	PKSEMAPHORE						pSemaphoreObj,
	OUT	PULONG							pSignalState
	);
	
KERNELAPI
VOID
KeReleaseSemaphore(
	IN	PKSEMAPHORE						pSemaphoreObj,
	IN	BOOL							bWait
	);
	
//
KERNELAPI
VOID
KeInitializeEvent(
	IN	PKEVENT							pEventObj,
	IN	KEVENT_TYPE						EventType,
	IN	BOOL							State
	);
	
KERNELAPI
BOOL
KeReadStateEvent(
	IN	PKEVENT							pEventObj,
	OUT	PULONG							pSignalState
	);
	
KERNELAPI
VOID
KeClearEvent(
	IN	PKEVENT							pEventObj,
	IN	BOOL							bWait
	);

KERNELAPI
VOID
KeSetEvent(
	IN	PKEVENT							pEventObj,
	IN	BOOL							bWait
	);
	
//
KERNELAPI
VOID
KeInitializeTimer(
	IN	PKTIMER							pTimerObj
	);
	
KERNELAPI
BOOL
KeSetTimer(
	IN	PKTIMER							pTimerObj,
	IN	ULONG							Timeout // milli seconds
	);
	
KERNELAPI
BOOL
KeCancelTimer(
	IN	PKTIMER							pTimerObj
	);

	
//////////////////////////////////////////////////////////////////////////////////////////
//                                  Memory Manager API                                  //
//////////////////////////////////////////////////////////////////////////////////////////
KERNELAPI
PVOID
MmAllocate(
	IN	ULONG							NumOfBytes
	);

KERNELAPI
VOID
MmFree(
	IN	PVOID							pMemAddr
	);


//////////////////////////////////////////////////////////////////////////////////////////
//                                    Task Manager API                                  //
//////////////////////////////////////////////////////////////////////////////////////////
typedef ULONG							TASK_PRIORITY, *PTASK_PRIORITY;

#define TASK_PRIORITY_LOWEST			7
#define TASK_PRIORITY_BELOW_NORMAL		4
#define TASK_PRIORITY_NORMAL			3
#define TASK_PRIORITY_ABOVE_NORMAL		2
#define TASK_PRIORITY_HIGHEST			0

typedef VOID (*PTASK_START_ROUTINE)(PVOID pContext);

KERNELAPI
HANDLE
TmCreateTask(
	IN	ULONG							StackSize,		// 0 = default
	IN	PTASK_START_ROUTINE				pStartRoutine,
	IN	PVOID							pParameter,
	IN	BOOL							bSuspend
	);

KERNELAPI
HANDLE
TmGetCurrentTask(VOID);

KERNELAPI
BOOL
TmSuspendTask(
	IN	HANDLE							TaskHandle
	);

KERNELAPI
BOOL
TmResumeTask(
	IN	HANDLE							TaskHandle
	);

KERNELAPI
BOOL
TmGetTaskPriority(
	IN	HANDLE							TaskHandle,
	OUT	PTASK_PRIORITY					pPriority
	);

KERNELAPI
BOOL
TmSetTaskPriority(
	IN	HANDLE							TaskHandle,
	IN	TASK_PRIORITY					Priority
	);

KERNELAPI
BOOL
TmTerminateTask(
	IN	HANDLE							TaskHandle
	);


//////////////////////////////////////////////////////////////////////////////////////////
//                                  Runtime Library API                                 //
//////////////////////////////////////////////////////////////////////////////////////////
#define SET_BITS(data, mask)			( (data) |= (mask) )
#define CLEAR_BITS(data, mask)			( (data) &= (~(mask)) )

KERNELAPI
VOID
RtlInitializeList(
	IN	PLIST_ENTRY						pList
	);

KERNELAPI
ULONG
RtlGetListCount(
	IN	PLIST_ENTRY						pList
	);

KERNELAPI
VOID
RtlInsertToListHead(
	IN	PLIST_ENTRY						pList,
	IN	PLIST_ENTRY						pEntry
	);

KERNELAPI
VOID
RtlInsertToListTail(
	IN	PLIST_ENTRY						pList,
	IN	PLIST_ENTRY						pEntry
	);

KERNELAPI
VOID
RtlRemoveEntryFromList(
	IN	PLIST_ENTRY						pEntry
	);


#endif /* #ifndef _DDK_HEADER_FILE_ */
