#include "ddk.h"
#include "gbadef.h"
#include "engfont8x16.h"

/*
 * definition
 */
#define MAX_CHARS				(GBA_DISP_WIDTH/8)		// 30 chars
#define MAX_LINES				(GBA_DISP_HEIGHT/16)	// 10 lines

//////////////////////////////////////////////////////////////////////////////////////////
KERNELAPI
BOOL
DbgColorPrintXY(
	IN	USHORT							TextColor,
	IN	USHORT							BkColor,
	IN	PSTR							pStr,
	IN	ULONG							XPos,
	IN	ULONG							YPos
	)
{
    PUSHORT		pt_video = (PUSHORT)GBA_DISP_VMEM_ADDR;
    ULONG		pitch;
	ULONG		x_pos;
	ULONG		y_pos;

    // init
	x_pos = XPos;
	y_pos = YPos;
    pitch = (240*2);
    
    while( *pStr )
    {
		UCHAR	ch;
        ULONG	y;
        
        // get char
        ch = (UCHAR)*pStr++;
        
        // is new line?
        if( ch == '\n' )
        {
            x_pos = 0;
            y_pos++;
            goto $adjust;
        }
        
        // print char
        for(y=0; y<16; y++)
        {
            int		x;
            ULONG	bitmap;
            PUSHORT	pt_tar_mem = NULL;
            
            // get
            bitmap = (ULONG)m_8x16EngFont[ch][y];
	
	        // get target start mem
	        pt_tar_mem
	        	= (PUSHORT)((ULONG)pt_video + (pitch*(y+(y_pos<<4))) + (x_pos<<4));
	
            // draw x
            for(x=7; x>=0; x--)
            {
                if( bitmap & (1<<x) )
                {
                    pt_tar_mem[7-x] = TextColor;
                }
                else
                {
                    pt_tar_mem[7-x] = BkColor;
                }
            }
        }
        
        // increase x_offset
        x_pos++;
        
        // adjust
$adjust:
        if( x_pos >= MAX_CHARS )
        {
            x_pos = 0;
            y_pos++;
        }
		if( y_pos >= MAX_LINES )
		{
			y_pos = (MAX_LINES-1);
		}
    }

    return TRUE;
}

KERNELAPI
BOOL
DbgPrint(
	IN	PSTR							pStr
	)
{
	return DbgColorPrintXY(RGB(255,255,255), RGB(0,0,0), pStr, 0, 0);
}

KERNELAPI
BOOL
DbgColorPrint(
	IN	USHORT							TextColor,
	IN	USHORT							BkColor,
	IN	PSTR							pStr
	)
{
	return DbgColorPrintXY(TextColor, BkColor, pStr, 0, 0);
}

KERNELAPI
BOOL
DbgPrintXY(
	IN	PSTR							pStr,
	IN	ULONG							XPos,
	IN	ULONG							YPos
	)
{
	return DbgColorPrintXY(RGB(255,255,255), RGB(0,0,0), pStr, XPos, YPos);
}

KERNELAPI
BOOL
DbgClearVideoScreen(VOID)
{
    return TRUE;
}

//////////////////////////////////////////////////////////////////////////////////////////
KERNELAPI
BOOL
DbgSetDebugVideoMode(VOID)
{
	// set mode
	GBA_DISP_REG_CNT = (3 | GBA_DISP_BG2_ENABLE);

	return TRUE;
}
