#ifndef _GBA_DEFINITION_HEADER_FILE_
#define _GBA_DEFINITION_HEADER_FILE_


/*
 * include files
 */
#include "types.h"


/*
 * system definition
 */
#define GBA_CPU_CLOCK			16777216 // 16.78MHz (2^24)

#define GBA_MODE_USER			0x10
#define GBA_MODE_FIQ			0x11
#define GBA_MODE_IRQ			0x12
#define GBA_MODE_SUPERVISOR		0x13
#define GBA_MODE_ABORT			0x17
#define GBA_MODE_UNDEFINED		0x1b
#define GBA_MODE_SYSTEM			0x1f


/*
 * video definition
 */
#define RGB(r,g,b)				((r&0x1f)+((g&0x1f)<<5)+((b&0x1f)<<10))

#define GBA_DISP_REG_CNT		( *(volatile ULONG*)0x04000000 )
#define GBA_DISP_BG2_ENABLE		0x400

#define GBA_DISP_VMEM_ADDR		0x06000000
#define GBA_DISP_WIDTH			240
#define GBA_DISP_HEIGHT			160


/*
 * interrupt definition
 */
#define GBA_INT_HANDLER_ADDR	( *(volatile ULONG*)0x03007ffc )
 
#define GBA_INT_REG_IME			( *(volatile USHORT*)0x04000208 )
#define GBA_INT_REG_IE			( *(volatile USHORT*)0x04000200 )
#define GBA_INT_REG_IF			( *(volatile USHORT*)0x04000202 )

#define GBA_INT_VBLANK			0x0001
#define GBA_INT_HBLANK			0x0002
#define GBA_INT_VCOUNT			0x0004
#define GBA_INT_TIMER0			0x0008
#define GBA_INT_TIMER1			0x0010
#define GBA_INT_TIMER2			0x0020
#define GBA_INT_TIMER3			0x0040
#define GBA_INT_COM				0x0080
#define GBA_INT_DMA0			0x0100
#define GBA_INT_DMA1			0x0200
#define GBA_INT_DMA2			0x0400
#define GBA_INT_DMA3			0x0800
#define GBA_INT_BUTTON			0x1000
#define GBA_INT_CART			0x2000

#define GBA_INT_TOTAL_NUM		14


/*
 * timer definition
 */
#define GBA_TMR_REG_TM0CNT		( *(volatile USHORT*)0x04000102 )
#define GBA_TMR_REG_TM1CNT		( *(volatile USHORT*)0x04000106 )
#define GBA_TMR_REG_TM2CNT		( *(volatile USHORT*)0x0400010a )
#define GBA_TMR_REG_TM3CNT		( *(volatile USHORT*)0x0400010e )

#define GBA_TMR_REG_TM0D		( *(volatile USHORT*)0x04000100 )
#define GBA_TMR_REG_TM1D		( *(volatile USHORT*)0x04000104 )
#define GBA_TMR_REG_TM2D		( *(volatile USHORT*)0x04000108 )
#define GBA_TMR_REG_TM3D		( *(volatile USHORT*)0x0400010c )
 
#define GBA_TMR_FREQ_SYSTEM		0x00
#define GBA_TMR_FREQ_64			0x01
#define GBA_TMR_FREQ_256		0x02
#define GBA_TMR_FREQ_1024		0x03
#define GBA_TMR_OVERFLOW		0x40
#define GBA_TMR_ENABLE			0x80

/*
 * keypad definition
 */
#define GBA_KEY_REG_CNT			( *(volatile USHORT*)0x04000132 )
#define GBA_KEY_REG_FLAG		( *(volatile USHORT*)0x04000130 )

/*
 * sound definition
 */
#define GBA_SND_REG_CNT0_L		( *(volatile USHORT*)0x04000080 )
#define GBA_SND_REG_CNT0_H		( *(volatile USHORT*)0x04000082 )
#define GBA_SND_REG_CNT1		( *(volatile USHORT*)0x04000084 )

/*
 * DMA definition
 */
#define GBA_DMA_REG_DM1SAD		( *(volatile ULONG*)0x040000BC )
#define GBA_DMA_REG_DM1DAD		( *(volatile ULONG*)0x040000C0 )
#define GBA_DMA_REG_DM1CNT		( *(volatile ULONG*)0x040000C4 )
#define GBA_DMA_REG_DM1CNT_H	( *(volatile USHORT*)0x040000C6 )


#endif /* #ifndef _GBA_DEFINITION_HEADER_FILE_ */
