#include "ddk.h"

/*
 * assembly functions
 */
extern ULONG HalpIrqHandler(VOID);

/*
 * variables
 */
static PIRQ_HANDLER	m_IrqHandlerTable[GBA_INT_TOTAL_NUM];


//////////////////////////////////////////////////////////////////////////////////////////
static
ULONG
HalpGetIrqTableNumber(
	IN	ULONG					IrqNumber
	)
{
    ULONG	table_num;
    for(table_num=0; !(IrqNumber & 0x01); IrqNumber>>=1, table_num++)
    	;
	return table_num;
}

//////////////////////////////////////////////////////////////////////////////////////////
VOID
HalpIrqHandlerMain(
	IN	ULONG					InterruptFlag
	)
{
    ULONG			table_num;
    PIRQ_HANDLER	pt_handler = NULL;

    // get irq handler
    table_num	= HalpGetIrqTableNumber(InterruptFlag);
    pt_handler	= m_IrqHandlerTable[table_num];

    // call handler
    if( pt_handler )
    {
        pt_handler();
	}

	// clear IF for next time
	GBA_INT_REG_IF |= (1 << table_num);
}

//////////////////////////////////////////////////////////////////////////////////////////
KERNELAPI
BOOL
HalInstallIrqHandler(
	IN	ULONG							IrqNumber,
	IN	PIRQ_HANDLER					pIrqHandler,		// can be NULL when delete!
	OUT	PIRQ_HANDLER					*ppOldIrqHandler	// optional
	)
{
    // check parameter
    if( IrqNumber == 0 || IrqNumber == GBA_INT_TIMER3 ) { return FALSE; }
    
    // do!
	DISABLE_INTERRUPTS();
    {
        ULONG	table_num;
        
	    // get irq table_num
	    table_num = HalpGetIrqTableNumber(IrqNumber);
		
		// save old one
		if( ppOldIrqHandler )
		{
		    *ppOldIrqHandler = m_IrqHandlerTable[ table_num ];
		}
		
		// replace!
		m_IrqHandlerTable[table_num] = pIrqHandler;
		
		// enable its interrupt mask bit
		GBA_INT_REG_IE |= IrqNumber;
	}
	ENABLE_INTERRUPTS();
	
	return TRUE;
}
	
//////////////////////////////////////////////////////////////////////////////////////////
KERNELAPI
BOOL
HalInitializeModule(VOID)
{
    // init irq table
    memset( m_IrqHandlerTable, 0, sizeof(m_IrqHandlerTable) );
    
    // init interrupt system
    do
	{
	    // init global ie mask
	    GBA_INT_REG_IE = 0;
	    
        // set irq handler
        GBA_INT_HANDLER_ADDR = (ULONG)HalpIrqHandler;
        
        // enable global interrupt mask, but interrupt is still disabled by ctrl irq bit
		ENABLE_INTERRUPTS();
	} while(FALSE);

    return TRUE;
}
